@@ -1,11 +1,13 @@
 """
 Module GW_helper.py
 ===================
-	Various routines for performing some tasks regarding GW signals (it uses lal and lalsimulation to work):
+	Various routines for performing some tasks regarding GW signals:
 		Mismatch computation
 			function compute_mismatch: computes the mismatch between two sets of wave (useful for seeing how important is reconstruction error)
 		Scalar product computation
 			function compute_scalar: computes the Wigner scalar product between two GW waveforms
+		Optimal mismatch computation:
+			function compute_optimal_mismatch: computes the optimal mismatch between two waves (i.e. by minimizing the mismatch w.r.t. the alignment)
 		Dataset creation Time Domain
 			function create_dataset_TD: creates a dataset of GW in time domain.
 		Dataset creation Frequency Domain
@@ -14,17 +16,12 @@ Module GW_helper.py
 #################
 
 import numpy as np
-import lalsimulation as lalsim
-import lal
 import os.path
 import matplotlib.pyplot as plt #debug
+import warnings
+import scipy.signal
 
 ################# Overlap related stuff
-def get_low_high_freq_index(flow, fhigh, df):
-	kmin = int(flow / df)
-	kmax = int(fhigh / df)
-	return kmin, kmax
-
 def overlap(amp_1, ph_1, amp_2, ph_2, df, low_freq = None, high_freq = None, PSD = None):
 		#it works only for input shapes (D,)
 	w1 = amp_1*np.exp(1j*ph_1)
@@ -102,11 +99,9 @@ compute_mismatch
 			ph_2 = np.reshape(ph_2, (1,ph_2.shape[0]))
 		D = amp_1.shape[1]
 		N = amp_1.shape[0]
-		#print(N,D)
 	else:
-		pass
-		#raise TypeError('Data don\'t have the same shape')
-		#return None
+		raise TypeError('Data don\'t have the same shape')
+		return None
 
 		#computing mismatch vector
 	df = 1. #nothing depends on df so we can set it arbitrarly
@@ -115,7 +110,7 @@ compute_mismatch
 	np.divide(F, div_factor, out = F)
 	return 1-F
 
-def compute_optimal_mismatch(h1,h2, return_F = True):
+def compute_optimal_mismatch(h1,h2, optimal = True, return_F = True):
 	"""
 compute_optimal_mismatch
 ========================
@@ -125,6 +120,7 @@ compute_optimal_mismatch
 	Input:
 		h1 (N,D)/(D,)	complex wave
 		h2 (N,D)/(D,)	complex wave
+		optimal			whether to optimize w.r.t. a constant phase
 		return_F		whether to reteurn mismatch or overlap
 	Output:
 		F_optimal (N,)/()		optimal mismatch/overlap
@@ -138,17 +134,14 @@ compute_optimal_mismatch
 	scalar = lambda h1_, h2_: np.sum(np.multiply(h1_,np.conj(h2_)), axis = 1)/h1_.shape[1] #(N,)
 
 	norm_factor = np.sqrt(np.multiply(scalar(h1,h1).real, scalar(h2,h2).real))
-	overlap = scalar(h1,h2) #(N,)
-	phi_optimal = np.angle(overlap) #(N,)
-	overlap = np.divide(scalar(h1,h2*np.exp(1j*phi_optimal)), norm_factor)
+	if optimal:
+		overlap = scalar(h1,h2) #(N,)
+		phi_optimal = np.angle(overlap) #(N,)
+	else:
+		phi_optimal = np.zeros(norm_factor.shape)
+	overlap = np.divide(scalar(h1,(h2.T*np.exp(1j*phi_optimal)).T), norm_factor)
 	overlap = overlap.real
 
-	#plt.plot(np.linspace(0,h1.shape[1],h1.shape[1]),np.squeeze(h1))
-	#plt.plot(np.linspace(0,h1.shape[1],h1.shape[1]),np.squeeze(h2)*np.exp(1j*phi_optimal))
-	#print(1-overlap, overlap)
-	#plt.show()
-
-
 	if return_F:
 		return 1-overlap, phi_optimal
 	if not return_F:
@@ -158,10 +151,10 @@ compute_optimal_mismatch
 
 ################# Dataset related stuff
 
-def create_dataset_TD(N_data, N_grid, filename = None,  t_coal = 0.5, q_range = (1.,5.), m2_range = None, s1_range = (-0.8,0.8), s2_range = (-0.8,0.8), t_step = 1e-5, approximant = "SEOBNRv2_opt", alpha = 0.35, path_TEOBResumS = None):
+def create_dataset_TD_TEOBResumS(N_data, N_grid, mode = (2,2),filename = None,  t_coal = 0.5, q_range = (1.,5.), m2_range = None, s1_range = (-0.8,0.8), s2_range = (-0.8,0.8), t_step = 1e-5, alpha = 0.35, path_TEOBResumS = None):
 	"""
-create_dataset_TD
-=================
+create_dataset_TD_TEOBResumS
+============================
 	Create a dataset for training a ML model to fit GW waveforms in time domain.
 	The dataset consists in 3 parameters theta=(q, spin1z, spin2z) associated to the waveform computed in frequency domain for a grid of N_grid points in the range given by the user.
 	More specifically, data are stored in 3 vectors:
@@ -171,10 +164,12 @@ create_dataset_TD
 	This routine add N_data data to filename if one is specified (if file is not empty it must contain data with the same N_grid); otherwise the datasets are returned as np vectors. 
 	All the waves are evaluated at a constant distance of 1Mpc. Values of q and m2 as well as spins are drawn randomly in the range given by the user: it holds m1 = q *m2 M_sun.
 	The waveforms are computed with a time step t_step; starting from a frequency f_min (set by the routine according to t_coal and m_tot). Waves are given in a rescaled time grid (i.e. t/m_tot) with N_grid points: t=0 occurs when at time of maximum amplitude. A higher density of grid points is placed in the post merger phase.
+	Dataset is generated either with an implementation of TEOBResumS (a path to a local installation of TEOBResumS should be provided).
 	Dataset can be loaded with load_dataset.
 	Input:
 		N_data				size of dataset
 		N_grid				number of grid points to evaluate
+		mode (l,m)			mode to generate and fill the dataset with				
 		filename			name of the file to save dataset in (If is None, nothing is saved on a file)
 		t_coal				time to coalescence to start computation from (measured in reduced grid)
 		q_range				tuple with range for random q values. if single value, q is kept fixed at that value
@@ -182,9 +177,9 @@ create_dataset_TD
 		spin_mag_max_1		tuple with range for random spin #1 values. if single value, s1 is kept fixed at that value
 		spin_mag_max_2		tuple with range for random spin #1 values. if single value, s2 is kept fixed at that value
 		t_step				time step to generate the wave with
-		approximant			string for the approximant model to be used (in lal convention)
+		approximant			string for the approximant model to be used (in lal convention; to be used only if lal ought to be used)
 		alpha				distorsion factor for time grid. (In range (0,1], when it's close to 0, more grid points are around merger)
-		path_TEOBResumS		path to a local installation of TEOBResumS with routine 'EOBRun_module' (only if TEOBResumS option is chosen)
+		path_TEOBResumS		path to a local installation of TEOBResumS with routine 'EOBRun_module' (if given, it overwrites the aprroximant entry)
 	Output:
 		if filename is given
 			None
@@ -197,6 +192,9 @@ create_dataset_TD
 	d=1.
 	inclination = 0.#np.pi/2.
 
+	if path_TEOBResumS is not None:
+		approximant = "TEOBResumS"
+
 	if approximant == "TEOBResumS":
 		#see https://bitbucket.org/eob_ihes/teobresums/src/development/ for the implementation of TEOBResumS
 		try:
@@ -206,8 +204,14 @@ create_dataset_TD
 		except:
 			raise RuntimeError("No valid imput source for module 'EOBRun_module' for TEOBResumS. Unable to continue.")
 	else:
+		try:
+			import lal
+			import lalsimulation as lalsim
+		except:
+			raise RuntimeError("Impossible to load lalsimulation: try pip install lalsuite")
 		LALpars = lal.CreateDict()
 		approx = lalsim.SimInspiralGetApproximantFromString(approximant)
+		
 
 		#checking if N_grid is fine
 	if not isinstance(N_grid, int):
@@ -218,9 +222,22 @@ create_dataset_TD
 	else:
 		D_theta = 3
 
+	if not isinstance(mode,tuple):
+		raise TypeError("Wrong kind of mode {} given. Expected a tuple (l,m)".format(mode))
+	modes = [mode]
+	modes_to_k = lambda modes:[int(x[0]*(x[0]-1)/2 + x[1]-2) for x in modes] # [(l,m)] -> [k]
+	k_modes = modes_to_k(modes)
+	if modes != [(2,2)]:
+		k_modes.append(1) #22 modes is always computed
+	print("Generating mode: "+str(modes[0]))
+
 		#creating time_grid
-	t_end = 5.2e-4 #estimated maximum time for ringdown: WF will be killed after that time
-	#alpha = 0.35 #exponent for "time distorsion"
+	if modes == [(2,2)]:
+		t_end = 5.2e-4 #estimated maximum time for ringdown: WF will be killed after that time
+	elif modes == [(2,1)] or modes == [(3,3)]:
+		t_end = 1e-6
+	else:
+		t_end = 3e-5 #only for HMs. You should find a better way to set this number...
 	time_grid = np.linspace(-np.power(np.abs(t_coal), alpha), np.power(t_end, alpha), N_grid)
 	time_grid = np.multiply( np.sign(time_grid) , np.power(np.abs(time_grid), 1./alpha))
 
@@ -241,7 +258,7 @@ create_dataset_TD
 			print("New file ", filename, " created")
 			freq_header = np.concatenate((np.zeros((3,)), time_grid, time_grid) )
 			freq_header = np.reshape(freq_header, (1,len(freq_header)))
-			np.savetxt(filebuff, freq_header, header = "row: theta "+str(D_theta)+" | amp (1,"+str(N_grid)+")| ph (1,"+str(N_grid)+")\nN_grid = "+str(N_grid)+" | t_coal ="+str(t_coal)+" | t_step ="+str(t_step)+" | q_range = "+str(q_range)+" | m2_range = "+str(m2_range)+" | s1_range = "+str(s1_range)+" | s2_range = "+str(s2_range), newline = '\n')
+			np.savetxt(filebuff, freq_header, header = "# row: theta "+str(D_theta)+" | amp (1,"+str(N_grid)+")| ph (1,"+str(N_grid)+")\n# N_grid = "+str(N_grid)+" | t_coal ="+str(t_coal)+" | t_step ="+str(t_step)+" | q_range = "+str(q_range)+" | m2_range = "+str(m2_range)+" | s1_range = "+str(s1_range)+" | s2_range = "+str(s2_range), newline = '\n')
 		else:
 			filebuff = open(filename,'a')
 
@@ -251,8 +268,8 @@ create_dataset_TD
 		theta_vector = np.zeros((N_data,D_theta))
 
 	for i in range(N_data): #loop on data to be created
-		if i%50 == 0 and i != 0:
-		#if i%1 == 0 and i != 0:
+		if i%100 == 0 and i != 0:
+		#if i%1 == 0 and i != 0: #debug
 			print("Generated WF ", i)
 
 			#setting value for data
@@ -283,32 +300,9 @@ create_dataset_TD
 		f_min = .9* ((151*(t_coal_freq)**(-3./8.) * (((1+q)**2)/q)**(3./8.))/(m1+m2))
 		 #in () there is the right scaling formula for frequency in order to get always the right reduced time
 		 #this should be multiplied by a prefactor (~1) for dealing with some small variation due to spins
+		#print(f_min, k_modes) #DEBUG
 
 			#getting the wave
-		if approximant != "TEOBResumS": #using lal to create WFs
-			hptilde, hctilde = lalsim.SimInspiralChooseTDWaveform( #where is its definition and documentation????
-				m1*lalsim.lal.MSUN_SI, #m1
-				m2*lalsim.lal.MSUN_SI, #m2
-				0., 0., spin1z, #spin vector 1
-				0., 0., spin2z, #spin vector 2
-				d*1e6*lalsim.lal.PC_SI, #distance to source
-				inclination, #inclination
-				0., #phi ref
-				0., #longAscNodes
-				0., #eccentricity
-				0., #meanPerAno
-				t_step, # time incremental step
-				f_min, # lowest value of time
-				f_min, #some reference value of time (??)
-				lal.CreateDict(), #some lal dictionary
-				approx #approx method for the model
-			)
-			#print(f_min, t_step)#debug
-			#print(m1,m2, spin1z,spin2z) #debug
-
-			h_p, h_c = np.array(hptilde.data.data), np.array(hctilde.data.data) #complex waveform
-			time_full = np.linspace(0.0, hptilde.data.length*t_step, hptilde.data.length) #time grid at which wave is computed
-
 		if approximant == "TEOBResumS": #using TEOBResumS
 			pars = {
 				'M'                  : m1+m2,
@@ -318,41 +312,33 @@ create_dataset_TD
 				'chi1'               : spin1z,
 				'chi2'               : spin2z,
 				'domain'             : 0,      # TD
-				'arg_out'            : 0,      # Output hlm/hflm. Default = 0
-				'use_mode_lm'        : [1],      # List of modes to use/output through EOBRunPy
+				'arg_out'            : 1,      # Output hlm/hflm. Default = 0
+				'use_mode_lm'        : k_modes,      # List of modes to use/output through EOBRunPy
 				'srate_interp'       : 1./t_step,  # srate at which to interpolate. Default = 4096.
 				'use_geometric_units': 0,      # Output quantities in geometric units. Default = 1
 				'initial_frequency'  : f_min,   # in Hz if use_geometric_units = 0, else in geometric units
-				'interp_uniform_grid': 2,      # Interpolate mode by mode on a uniform grid. Default = 0 (no interpolation)
+				'interp_uniform_grid': 0,      # Interpolate mode by mode on a uniform grid. Default = 0 (no interpolation)
 				'distance': d,
 				'inclination':inclination,
 			}
-			time_full, h_p, h_c = EOBRun_module.EOBRunPy(pars)
-
+			time_full, h_p, h_c, hlm = EOBRun_module.EOBRunPy(pars)
+			
 		if isinstance(m2_range, tuple):
 			temp_theta = [m1, m2, spin1z, spin2z]		
 		else:
 			temp_theta = [m1/m2, spin1z, spin2z]
 
-		temp_amp = np.sqrt(np.square(h_p)+np.square(h_c))
-		temp_ph = np.unwrap(np.arctan2(h_c,h_p))
+		temp_amp = hlm[str(k_modes[0])][0]
+		temp_ph = hlm[str(k_modes[0])][1]
 
-		time_full = (time_full - time_full[np.argmax(temp_amp)])/(m1+m2) #grid is scaled to standard grid
+		argpeak = locate_peak(hlm['1'][0]) #aligned at the peak of the 22
+		t_peak = time_full[argpeak]
+		time_full = (time_full - t_peak)/(m1+m2) #grid is scaled to standard grid
 			#setting waves to the chosen std grid
 		temp_amp = np.interp(time_grid, time_full, temp_amp)
 		temp_ph = np.interp(time_grid, time_full, temp_ph)
 
-			#here you need to decide what is better
-		#temp_ph = temp_ph - temp_ph[0] #all phases are shifted by a constant to make every wave start with 0 phase
-		id0 = np.where(time_grid == 0)[0]
-		temp_ph = temp_ph - temp_ph[id0] #all phases are shifted by a constant to make every wave start with 0 phase at t=0 (i.e. at maximum amplitude)
-
-			#removing spourious gaps (if present) (do I need it??)
-		(index,) = np.where(temp_amp/np.max(temp_amp) < 1e-5) #there should be a way to choose the right threshold...
-		if len(index) >0:
-			print("Wave killed")
-			temp_amp[index] = temp_amp[index[0]-1]
-			temp_ph[index] = temp_ph[index[0]-1]
+		temp_ph = temp_ph - temp_ph[0] #all phases are shifted by a constant to make sure every wave has 0 phase at beginning of grid
 
 		if filename is None:
 			amp_dataset[i,:] = temp_amp  #putting waveform in the dataset to return
@@ -370,6 +356,7 @@ create_dataset_TD
 		filebuff.close()
 		return None
 
+
 def generate_waveform(m1,m2, s1=0.,s2 = 0.,d=1., iota = 0.,phi_0=0., t_coal = 0.4, t_step = 5e-5, f_min = None, t_min = None, verbose = False, approx = "SEOBNRv2_opt"):
 	"""
 generate_waveform
@@ -387,6 +374,8 @@ generate_waveform
 		h_p (N,D)	plus polarization of the wave
 		h_c (N,D)	cross polarization of the wave
 	"""
+	import lal
+	import lalsimulation as lalsim
 	q = m1/m2
 	mtot = (m1+m2)#*lal.MTSUN_SI
 	mc = (m1*m2)**(3./5.)/(m1+m2)**(1./5.)
@@ -435,23 +424,28 @@ generate_waveform
 
 	return times[arg:], h_p[arg:], h_c[arg:]
 
-def generate_waveform_TEOBResumS(m1,m2, s1=0.,s2 = 0.,d=1., iota = 0., t_coal = 0.4, t_step = 5e-5, f_min = None, t_min = None, verbose = False, path_TEOBResumS = None):
+def generate_waveform_TEOBResumS(m1,m2, s1=0.,s2 = 0.,d=1., iota = 0., t_coal = 0.4, t_step = 5e-5, f_min = None, t_min = None,
+modes = [(2,2)], verbose = False, path_TEOBResumS = None):
 	"""
 generate_waveform
 =================
-	Wrapper to lalsimulation.SimInspiralChooseTDWaveform() to generate a single waveform. Wave is not preprocessed.
+	Wrapper to EOBRun_module.EOBRunPy() to generate a single waveform with TEOBResumS. Wave is not preprocessed.
 	Input:
 		m1,m2,s1,s2,d,iota,phi_0	orbital parameters
 		t_coal						approximate time to coalescence in reduced grid (ignored if f_min or t_min is set)
 		t_step						EOB integration time to be given to lal
 		f_min						starting frequency in Hz (if None, it will be determined by t_coal; ignored if t_min is set)
 		t_min						starting time in s (if None, t_coal will be returned)
+		modes 						list of modes to be included in the WFs
 		verbose						whether to print messages for each wave...
 	Output:
 		times (D,)	times at which wave is evaluated
 		h_p (D,)	plus polarization of the wave
 		h_c (D,)	cross polarization of the wave
+		t_m 		time at which amplitude peaks
 	"""
+	modes_to_k = lambda modes:[int(x[0]*(x[0]-1)/2 + x[1]-2) for x in modes] # [(l,m)] -> [k]
+
 	if path_TEOBResumS is None: #very ugly but useful
 		path_TEOBResumS = '/home/stefano/Desktop/Stefano/scuola/uni/tesi_magistrale/code/TEOBResumS/Python'
 	import sys
@@ -474,6 +468,8 @@ generate_waveform
 
 	if verbose:
 		print("Generating wave @: ",m1,m2,s1,s2,d, iota)
+	
+	k_modes = modes_to_k(modes)
 
 	pars = {'M'                  : m1+m2,
 			'q'                  : m1/m2,
@@ -482,19 +478,26 @@ generate_waveform
 			'chi1'               : s1,
 			'chi2'               : s2,
 			'domain'             : 0,      # TD
-			'arg_out'            : 0,      # Output hlm/hflm. Default = 0
-			'use_mode_lm'        : [1],      # List of modes to use/output through EOBRunPy
+			'arg_out'            : 1,     # Output hlm/hflm. Default = 0
+			'use_mode_lm'        : k_modes,      # List of modes to use/output through EOBRunPy
 			'srate_interp'       : 1./t_step,  # srate at which to interpolate. Default = 4096.
 			'use_geometric_units': 0,      # Output quantities in geometric units. Default = 1
 			'initial_frequency'  : f_min,   # in Hz if use_geometric_units = 0, else in geometric units
 			'interp_uniform_grid': 2,      # Interpolate mode by mode on a uniform grid. Default = 0 (no interpolation)
 			'distance': d,
 			'inclination': iota,
+			#'nqc':2, #{"no", "auto", "manual"}
+			#'nqc_coefs_flx': 2, # {"none", "nrfit_nospin20160209", "fit_spin_202002", "fromfile"}
+			#'nqc_coefs_hlm':0
 		}
-	times, h_p, h_c = EOBRun_module.EOBRunPy(pars)
-	amp = np.sqrt(h_p**2+h_c**2)
+	times, h_p, h_c, hlm = EOBRun_module.EOBRunPy(pars)
 
-	t_m =  times[np.argmax(amp)]
+	try:
+		argpeak = locate_peak(hlm['1'][0])
+	except:
+		amp = np.sqrt(h_p**2+h_c**2)
+		argpeak = locate_peak(amp)
+	t_m =  times[argpeak]
 	times = times - t_m
 
 	if t_min is not None:
@@ -502,164 +505,25 @@ generate_waveform
 	else:
 		arg=0
 
-	return times[arg:], h_p[arg:], h_c[arg:]
+	return times[arg:], h_p[arg:], h_c[arg:], hlm, t_m
 
 
-
-
-def create_dataset_FD(N_data, N_grid = None, filename = None, q_range = (1.,5.), m2_range = 20., s1_range = (-0.8,0.8), s2_range = (-0.8,0.8), log_space = True, f_high = 2000, f_step = 1e-2, f_max = None, f_min =None, approximant = "IMRPhenomPv2"):
+def locate_peak(amp, start = 0.3):
 	"""
-	Create a dataset for training a ML model to fit GW waveforms in frequency domain.
-	The dataset consists in 3 parameters theta=(q, spin1z, spin2z) associated to the waveform computed in frequency domain for a grid of N_grid points in the range given by the user.
-	More specifically, data are stored in 3 vectors:
-		theta_vector	vector holding source parameters q, spin1, spin2
-		amp_vector		vector holding amplitudes for each source evaluated at some N_grid equally spaced points
-		ph_vector		vector holding phase for each source evaluated at some N_grid equally spaced points
-	This routine add N_data data to filename if one is specified (if file is not empty it must contain data with the same N_grid); otherwise the datasets are returned as np vectors. 
-	All the waves are evaluated at a constant distance of 1Mpc. Values of q and m2 are drawn randomly in the range given by the user: it holds m1 = q *m2 M_sun.
-	The waveforms are computed from f_low = 15 to f_high with a step f_step and then evaluated at some N_grid grid points equally spaced in range [f_min, f_max]
-	Dataset can be loaded with load_dataset
+	Given a time grid and an amplitude of the mode, it returns the peak of the amplitude.
 	Input:
-		N_data			size of dataset
-		N_grid			number of points to be sampled in the grid (if None every point generated is saved)
-		filename		name of the file to save dataset in (If is None, nothing is saved on a file)
-		q_range			tuple with range for random q values. if single value, q is kept fixed at that value
-		m2_range		tuple with range for random m2 values. if single value, m2 is kept fixed at that value
-		spin_mag_max_1	tuple with range for random spin #1 values. if single value, s1 is kept fixed at that value
-		spin_mag_max_2	tuple with range for random spin #1 values. if single value, s2 is kept fixed at that value
-		log_space		whether grid should be computed in logspace
-		f_high			highest frequency to compute
-		f_step			step considered for computation of waveforms
-		f_max			maximum frequency returned to the user (if None is the same as f_max)
-		f_min			minimum frequency returned to the user (if None is the same as f_low = 15)
-		approximant	string for the approximant model to be used (in lal convention)
+		amp (D,)		amplitude
+		start			points to be skipped at begining of the amplitude (in fraction of total points)
 	Output:
-		if filename is given
-			None
-		if filename is not given
-			theta_vector (N_data,3)		vector holding ordered set of parameters used to generate amp_dataset and ph_dataset
-			amp_dataset (N_data,N_grid)	dataset with amplitudes
-			ph_dataset (N_data,N_grid)	dataset with phases
-			frequencies (N_grid,)		vector holding frequencies at which waves are evaluated
+		argpeak			index at which the amplitude has a peak
 	"""
-	if f_max is None:
-		f_max = f_high
-	if f_min is None:
-		f_min = 1.
-	f_low = f_min
-	K = int((f_max-f_min)/f_step) #number of data points to be taken from the returned vector
-	if N_grid is None:
-		N_grid = K
-	full_freq = np.arange(f_low, f_max, f_step) #full frequency vector as returned by lal
-	d=1.
-	LALpars = lal.CreateDict()
-	approx = lalsim.SimInspiralGetApproximantFromString(approximant)
+	assert amp.ndim == 1
+	id_start = int(len(amp)*start)
+	extrema = scipy.signal.argrelextrema(np.abs(amp[id_start:]), np.greater)
+	argpeak = extrema[0][0]+id_start
+	return argpeak
 
-		#allocating storage for temp vectors to save a single WF
-	temp_amp = np.zeros((N_grid,))
-	temp_ph = np.zeros((N_grid,))
-	temp_theta = np.zeros((3,))
 
-		#setting frequencies to be returned to user
-	if log_space:
-		frequencies = np.logspace(np.log10(f_min), np.log10(f_max), N_grid)
-	else:
-		frequencies = np.linspace(f_min, f_max, N_grid)
-		#freq_to_choose = np.arange(0, K, K/N_grid).astype(int) #choosing proper indices s.t. dataset holds N_grid points
-		#frequencies = full_freq[freq_to_choose] #setting only frequencies to be chosen
-
-	if filename is not None: #doing header if file is empty - nothing otherwise
-		if not os.path.isfile(filename): #file doesn't exist: must be created with proper header
-			filebuff = open(filename,'w')
-			print("New file ", filename, " created")
-			freq_header = np.concatenate((np.zeros((3,)), frequencies, frequencies) )
-			freq_header = np.reshape(freq_header, (1,len(freq_header)))
-			np.savetxt(filebuff, freq_header, header = "row: theta 3 | amp "+str(frequencies.shape[0])+"| ph "+str(frequencies.shape[0])+"\nN_grid = "+str(N_grid)+" | f_step ="+str(f_step)+" | q_range = "+str(q_range)+" | s1_range = "+str(s1_range)+" | s2_range = "+str(s2_range), newline = '\n')
-		else:
-			filebuff = open(filename,'a')
-
-	if filename is None:
-		amp_dataset = np.zeros((N_data,N_grid)) #allocating storage for returning data
-		ph_dataset = np.zeros((N_data,N_grid))
-		theta_vector = np.zeros((N_data,3))
-
-	for i in range(N_data): #loop on data to be created
-		if i%100 == 0 and i != 0:
-			print("Generated WF ", i)
-
-			#setting value for data
-		if isinstance(m2_range, tuple):
-			m2 = np.random.uniform(m2_range[0],m2_range[1])
-		else:
-			m2 = m2_range
-		if isinstance(q_range, tuple):
-			m1 = np.random.uniform(q_range[0],q_range[1]) * m2
-		else:
-			m1 = q_range * m2
-		if isinstance(s1_range, tuple):
-			spin1z = np.random.uniform(s1_range[0],s1_range[1])
-		else:
-			spin1z = s1_range
-		if isinstance(s2_range, tuple):
-			spin2z = np.random.uniform(s2_range[0],s2_range[1])
-		else:
-			spin2z = s2_range
-
-			#debug!!!
-		if m1/m2 >4.6 and (np.abs(spin1z) >0.8 or np.abs(spin2z) > 0.8):
-			continue
-
-			#getting the wave
-		hptilde, hctilde = lalsim.SimInspiralChooseFDWaveform( #where is its definition and documentation????
-			m1*lalsim.lal.MSUN_SI, #m1
-			m2*lalsim.lal.MSUN_SI, #m2
-			0., 0., spin1z, #spin vector 1
-			0., 0., spin2z, #spin vector 2
-			d*1e6*lalsim.lal.PC_SI, #distance to source
-			0., #inclination
-			0., #phi ref
-			0., #longAscNodes (for precession)
-			0., #eccentricity
-			0., #meanPerAno (for precession)
-			f_step, # frequency incremental step
-			f_low, # lowest value of frequency
-			f_high, # highest value of frequency
-			f_low, #some reference value of frequency (??)
-			LALpars, #some lal dictionary
-			approx #approx method for the model
-			)
-		h = np.array(hptilde.data.data)+1j*np.array(hctilde.data.data) #complex waveform
-		temp_theta = [m1/m2, spin1z, spin2z]
-		temp_amp = (np.abs(h)[int(f_min/f_step):int(f_max/f_step)].real)
-		temp_ph = (np.unwrap(np.angle(h))[int(f_min/f_step):int(f_max/f_step)].real)
-
-			#bringing waves on the chosen grid
-		temp_amp = np.interp(frequencies, full_freq, temp_amp)
-		temp_ph = np.interp(frequencies, full_freq, temp_ph)
-		#temp_ph = temp_ph[freq_to_choose]; temp_amp = temp_amp[freq_to_choose] #old version of code
-
-		temp_ph = temp_ph - temp_ph[0] #all frequencies are shifted by a constant to make the wave start at zero phase!!!! IMPORTANT
-
-			#removing spourious gaps (if present)
-		(index,) = np.where(temp_amp/temp_amp[0] < 5e-3) #there should be a way to choose right threshold...
-		if len(index) >0:
-			temp_ph[index] = temp_ph[index[0]-1]
-
-		if filename is None:
-			amp_dataset[i,:] = temp_amp  #putting waveform in the dataset to return
-			ph_dataset[i,:] =  temp_ph  #phase
-			theta_vector[i,:] = temp_theta
-
-		if filename is not None: #saving to file
-			to_save = np.concatenate((temp_theta,temp_amp, temp_ph))
-			to_save = np.reshape(to_save, (1,len(to_save)))
-			np.savetxt(filebuff, to_save)
-
-	if filename is None:
-		return theta_vector, amp_dataset.real, ph_dataset.real, frequencies
-	else:
-		filebuff.close()
-		return None
 
 def save_dataset(filename, theta_vector, dataset1, dataset, x_grid):
 	"""
@@ -679,7 +543,7 @@ def save_dataset(filename, theta_vector, dataset1, dataset, x_grid):
 	q_max = np.max(theta_vector[:,0])
 	spin_mag_max = np.max(np.abs(theta_vector[:,1:2]))
 	x_step = x_grid[1]-x_grid[0]
-	np.savetxt(filename, to_save, header = "row: theta 3 | amp "+str(amp_dataset.shape[1])+"| ph "+str(ph_dataset.shape[1])+"\nN_grid = "+str(x_grid.shape[0])+" | f_step ="+str(x_step)+" | q_max = "+str(q_max)+" | spin_mag_max = "+str(spin_mag_max), newline = '\n')
+	np.savetxt(filename, to_save, header = "# row: theta 3 | amp "+str(amp_dataset.shape[1])+"| ph "+str(ph_dataset.shape[1])+"\n# N_grid = "+str(x_grid.shape[0])+" | f_step ="+str(x_step)+" | q_max = "+str(q_max)+" | spin_mag_max = "+str(spin_mag_max), newline = '\n')
 	return
 
 
